package com.nisum.employeeservice.controller;

import org.springframework.web.bind.annotation.*;
import com.nisum.employeeservice.exception.ResourceNotFoundException;
import com.nisum.employeeservice.model.Employee;
import com.nisum.employeeservice.repository.EmployeeRepository;
import com.nisum.employeeservice.service.EmployeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


import java.util.List;

@CrossOrigin(origins = "*")
@RestController
@RequestMapping("/api/v1")

public class EmployeeController {
  @Autowired EmployeeService employeeService;
  @Autowired EmployeeRepository employeeRepository;

  // Return all Employee(s)
  @GetMapping("/employees")
  public List<Employee> getAllEmployess() {
    return employeeService.findAllEmployees();
  }

  // Create Employee
  @PostMapping("/employees")
  public Employee createEmployee(@RequestBody Employee employee) {
    return employeeService.addEmployee(employee);
  }

  // Get Employee by ID
  @GetMapping("/employees/{id}")
  public ResponseEntity<Employee> getEmployeeById(@PathVariable(value = "id") Long employeeId) throws ResourceNotFoundException {
    Employee employee = employeeService.getEmployeeById(employeeId)
            .orElseThrow(ResourceNotFoundException::new);
    return ResponseEntity.ok().body(employee);
  }

  // Update Employee by ID
  @PutMapping("/employees/{id}")
  public ResponseEntity<Employee> updateEmployee(@PathVariable(value = "id") Long id, @RequestBody Employee employeeInfo) throws ResourceNotFoundException {
    Employee employee = employeeService.getEmployeeById(id).orElseThrow(ResourceNotFoundException::new);
    employee.setFirstName(employeeInfo.getFirstName());
    employee.setLastName(employeeInfo.getLastName());
    Employee updatedEmployee = employeeRepository.save(employee);
    return ResponseEntity.ok().body(updatedEmployee);
  }

  // Delete Employee
  @DeleteMapping("/employees/{id}")
  public List<Employee> deleteEmployee(@PathVariable(value = "id") Long id) {
    employeeService.removeEmployee(id);
    return employeeService.findAllEmployees();
  }
}
